/*---------------------------------------------------------------------------*\
File
    initializeForceBalance.H

Description
    Initializes for force balance part of the solver
*   These parameters are read in only at the beginning of the simulation run
    and hence none of the parameters below can be modified during a simulation

\*---------------------------------------------------------------------------*/


    Info << endl << "Reading field cellMotionU" << endl;

volVectorField& cellDisplacement = const_cast<volVectorField&>
    (
        mesh.objectRegistry::lookupObject<volVectorField>
        (

	            "cellDisplacement"
        )
    );

    Info << "Reading field pointMotionU" << endl;

pointVectorField& pointDisplacement = const_cast<pointVectorField&>
    (
        mesh.objectRegistry::lookupObject<pointVectorField>
        (

	            "pointDisplacement"
        )
    );

    // Read the turbForceFoam dictionary object
    Info << "Reading the force balance dictionary" << endl;
    IOdictionary forceFoamDict
    (
    	IOobject
        (
            "forceFoamDict",
            runTime.constant(),
            mesh,
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        )
    );


    // Create the dictionary in which status data is saved during each write
    Info << "Reading force solver data" << endl;
    IOdictionary forceSolverData
    (
       IOobject
       (
          "forceSolverData",
          runTime.timeName(),
          mesh,
          IOobject::READ_IF_PRESENT,
          IOobject::AUTO_WRITE
       )
    );

    const dictionary& forceBalance = forceFoamDict.subDict("forceBalance");

    // Basic switch for enabling or disabling the module [One-time read]
    bool forceBalanceEnabled = false;

    if(forceBalance.found("forceBalanceEnabled"))
    {
       forceBalanceEnabled = Switch(forceBalance.lookup("forceBalanceEnabled"));
    }

    // Static variables for previous solve time, cumulative velocity and distance
    static scalar previousCalcTime_M = 0.0;
    static scalar previousCalcTime_F = 0.0;
    static vector calcVelocity = vector(0,0,0);
    static vector calcDistance = vector(0,0,0);
    static vector thetaVelty = vector(0,0,0);
    static vector thetaDiste = vector(0,0,0);

    // Read in the patches which constitute the moving parts of the mesh [One-time read]
    const wordList motionPatches(forceFoamDict.lookup("motionPatches"));

    if(forceBalanceEnabled)
    {
       Info << endl << "Force based solver enabled......." << endl << endl;

       // Extract the different patch types available within the mesh
       const wordList patchTypesFB=mesh.boundaryMesh().types();

       // Check if the motion patches are valid
       forAll(motionPatches,mPI)
       {
          label patchIndex = mesh.boundaryMesh().findPatchID(motionPatches[mPI]);
          if(patchIndex < 0)
          {
             FatalErrorIn(args.executable())
                         << motionPatches[mPI] << " not found in mesh definition!!" << endl
                         << exit(FatalError);
          }
    
          if(patchTypesFB[patchIndex] != "wall")
          {
             Info << "Warning....The patch " << motionPatches[mPI] << " is not of type wall...!!" << endl;
          }
       }       

       // Read in the current force solver Data if present, else, new start
       // Last absolute time at which mechanical system was solved
       if(forceSolverData.found("previousCalcTime"))
       {
//           previousCalcTime = readScalar(forceSolverData.lookup("previousCalcTime"));
//           Info << "Using last solved time [sec] = " << previousCalcTime_M << endl;
       }

       // Velocity calculated at last saved time
       if(forceSolverData.found("calcVelocity"))
       {
//           calcVelocity = readScalar(forceSolverData.lookup("calcVelocity"));
           Info << "Using last calculated velocity [m/s] = " << calcVelocity << endl;
       }

       // Position calculated at last saved time
       if(forceSolverData.found("calcDistance"))
       {
//           calcDistance = readScalar(forceSolverData.lookup("calcDistance"));
           Info << "Using last calculated distance [m] = " << calcDistance << endl;
       }


    }

    // Declare and initialize the variables used


    dimensionedScalar bodyInertia     = dimensionedScalar("binertia",dimensionSet(1,2,0,0,0,0,0),11.09);
    dimensionedScalar bodyMass        = dimensionedScalar("M",dimensionSet(1,0,0,0,0,0,0),136.51);

    dimensionedScalar gravityAccel    = dimensionedScalar("g",dimensionSet(0,1,-2,0,0,0,0),9.81);

    dimensionedScalar accelLim        = dimensionedScalar("accelLim",dimensionSet(0,1,-2,0,0,0,0),0.0);



    vector CgCenter = vector(0,0,0);

    vector gVector = vector(0,-1,0);
    vector dirVector1 = vector(1,0,0);
    vector dirVector2 = vector(0,1,0);
    vector dirVector3 = dirVector1 ^ dirVector2;

    vectorField tempRotation5 ;
    pointField  tempRotation6 ;

     vectorField tempCell;
     pointField tempPoint;
     vectorField tempCellE;
     pointField tempPointE;

     vectorField tempCelld;
     pointField tempPointd;
     vectorField tempCellr;
     pointField tempPointr;       


    forAll(motionPatches,mPI3)
    {     	
      label patchIndex = mesh.boundaryMesh().findPatchID(motionPatches[mPI3]);
      
      tempCell = mesh.C().boundaryField()[patchIndex];
      tempPoint = mesh.boundaryMesh()[patchIndex].localPoints();
    }

  p = rho*g.component(vector::Y)*mesh.C().component(vector::Y);//sant?
  p.correctBoundaryConditions();

